<?php
namespace DaReactions\Pages;
use DaReactions\Abstracts\AbstractAdminPage;
use DaReactions\Cache;
use DaReactions\Data;
use DaReactions\FileSystem;
use DaReactions\Utils;
use Exception;
/**
 *
 */
class AdminPageImportVotes extends AbstractAdminPage {
	private $upload_nonce = 'da_reactions_upload_csv';
	private $upload_nonce_name = 'da_reactions_upload_nonce_name';
	private $import_nonce = 'da_reactions_import_csv';
	private $import_nonce_name = 'da_reactions_import_nonce_name';
	private $valid_csv = false;
	private $total_records = [];
	private $valid_records = [];
	private $invalid_records = [];
	private $unregistered_types = [];
	private $unregistered_contents = [];
	private $unregistered_users = [];
	private $unregistered_reactions = [];
	private $registered_types = [];
	private $registered_contents = [];
	private $registered_users = [];
	private $registered_reactions = [];
	private $invalid_import_link = '';
	private $valid_import_link = '';
	private $uploaded_file_name = '';
	private $invalid_rows_file_name = '';
	private $valid_rows_file_name = '';
	private $upload_path = '/da-reactions/csv/';
	/**
	 * @throws Exception
	 */
	public function displayPage(): void {
		$upload_dir = wp_upload_dir();
		if ( isset( $_POST[ $this->import_nonce_name ] ) ) {
			check_admin_referer( $this->import_nonce, $this->import_nonce_name );
			$this->valid_rows_file_name = sanitize_text_field( $_POST['valid_import_file_name'] );
			$valid_import_path          = $upload_dir['basedir'] . $this->upload_path . $this->valid_rows_file_name;
			if (
				(
				$handle = fopen( $valid_import_path, 'rb' )
				) !== false
			) {
				while ( ( $row = fgetcsv( $handle ) ) !== false ) {
					$this->sanitizeRecord( $row );
				}
				if ( count( $this->valid_records ) < 1 ) {
					$this->renderImportComplete();
					return;
				}
				Cache::deleteAll();
				$importCount = 0;
				/// Init a new csv file
				$remaining_rows_file_name = Utils::generateRandomString() . '.csv';
				$remaining_rows_file_path = $upload_dir['basedir'] . $this->upload_path . $remaining_rows_file_name;
				if (
					(
					$remaining_rows_file_handle = fopen( $remaining_rows_file_path, 'wb' )
					) !== false
				) {
					foreach ( $this->valid_records as $record ) {
						$importCount ++;
						$importQuota = 5000;
						if ( $importCount > $importQuota ) {
							fputcsv(
								$remaining_rows_file_handle,
								array(
									$importCount,
									$record['resource_type'],
									$record['resource_id'],
									$record['emotion_id'],
									$record['user_id'],
									$record['user_token'],
									$record['user_ip'],
									$record['created_at']
								)
							);
						} else {
							Data::insertReactionIntoDB(
								$record['resource_id'],
								$record['resource_type'],
								$record['emotion_id'],
								$record['user_id'],
								$record['user_token'],
								$record['user_ip']
							);
						}
					}
				}
				fclose( $remaining_rows_file_handle );
				?>
                <p><?php echo sprintf( __( 'Importing from CSV, %d rows remaining.', 'da-reactions' ), count( $this->valid_records ) ); ?></p>
                <form id="importingForm" action="" method="POST">
					<?php
					wp_nonce_field( $this->import_nonce, $this->import_nonce_name );
					?>
                    <input
                            type="hidden"
                            name="valid_import_file_name"
                            value="<?php echo $remaining_rows_file_name; ?>"
                    />
                </form>
				<?php
				fclose( $handle );
			} else {
				?><p class="error"><?php _e( 'Cannot open imported file', 'da-reactions' ); ?></p><?php
			}
		} else if ( isset( $_POST[ $this->upload_nonce_name ] ) ) {
			check_admin_referer( $this->upload_nonce, $this->upload_nonce_name );
			if (
				$_FILES['uploadedCsv']['error'] === UPLOAD_ERR_OK &&
				is_uploaded_file( $_FILES['uploadedCsv']['tmp_name'] )
			) {
				if ( FileSystem::isCsv( $_FILES['uploadedCsv'] ) ) {
					$this->valid_csv              = true;
					$this->uploaded_file_name     = Utils::generateRandomString() . '.csv';
					$this->invalid_rows_file_name = Utils::generateRandomString() . '.csv';
					$this->valid_rows_file_name   = Utils::generateRandomString() . '.csv';
					/// Save to FileSystem
					$this->valid_import_link   = $upload_dir['baseurl'] . $this->upload_path . $this->valid_rows_file_name;
					$this->invalid_import_link = $upload_dir['baseurl'] . $this->upload_path . $this->invalid_rows_file_name;
					$valid_import_path   = $upload_dir['basedir'] . $this->upload_path . $this->valid_rows_file_name;
					$invalid_import_path = $upload_dir['basedir'] . $this->upload_path . $this->invalid_rows_file_name;
					$path_with_end_slash = $upload_dir['basedir'] . $this->upload_path;
					FileSystem::putContents(
						$path_with_end_slash,
						$this->uploaded_file_name,
						FileSystem::urlGetContents( $_FILES['uploadedCsv']['tmp_name'] )
					);
					/// Read file as CSV
					$this->total_records          = [];
					$this->valid_records          = [];
					$this->invalid_records        = [];
					$this->unregistered_types     = [];
					$this->unregistered_contents  = [];
					$this->unregistered_users     = [];
					$this->unregistered_reactions = [];
					$this->registered_types       = [];
					$this->registered_contents    = [];
					$this->registered_users       = [];
					$this->registered_reactions   = [];
					if (
						(
						$handle = fopen( $path_with_end_slash . $this->uploaded_file_name, 'rb' )
						) !== false
					) {
						while ( ( $row = fgetcsv( $handle ) ) !== false ) {
							$this->sanitizeRecord( $row );
						}
						fclose( $handle );
					}
					/// Write invalid and valid CSV files
					if (
						(
						$handle = fopen( $valid_import_path, 'wb' )
						) !== false
					) {
						fputcsv( $handle, array(
							'ID',
							'resource_type',
							'resource_id',
							'emotion_id',
							'user_id',
							'user_token',
							'user_ip',
							'created_at'
						) );
						foreach ( $this->valid_records as $row ) {
							fputcsv( $handle, array_slice( $row, 0, 8 ) );
						}
						fclose( $handle );
					}
					if (
						(
						$handle = fopen( $invalid_import_path, 'wb' )
						) !== false
					) {
						fputcsv( $handle, array(
							'ID',
							'resource_type',
							'resource_id',
							'emotion_id',
							'user_id',
							'user_token',
							'user_ip',
							'created_at',
							'error'
						) );
						foreach ( $this->invalid_records as $row ) {
							fputcsv( $handle, array_slice( $row, 0, 9 ) );
						}
						fclose( $handle );
					}
				} else {
					$this->valid_csv = false;
				}
				$this->renderImportForm();
			}
		} else {
			FileSystem::deleteAllCsv();
			$this->renderUploadForm();
		}
	}
	private function renderUploadForm(): void {
		?>
        <div class="wrap" id="admin_page_import_votes">
            <h1 class="wp-heading-inline">
				<?php _ex( 'Import votes', 'Import page title', 'da-reactions' ); ?>
            </h1>
            <div class="form-container">
                <form id="uploadForm" action="" method="POST" enctype="multipart/form-data">
					<?php
					wp_nonce_field( $this->upload_nonce, $this->upload_nonce_name );
					?>
                    <input type="file" name="uploadedCsv">
                    <p>&nbsp;</p>
                </form>
                <button id="uploadSubmit"><?php _ex( 'Upload', 'Upload CSV Form Submit', 'da-reactions' ); ?></button>
            </div>
        </div>
		<?php
	}
	private function renderImportComplete(): void {
		?>
        <div class="da-reactions-panel">
            <div class="da-reactions-panel-content">
                <h2>
					<?php _ex( 'Import complete', 'Upload result page title', 'da-reactions' ); ?>
                </h2>
                <p>
                    <a href="<?= admin_url( 'admin.php' ) ?>?page=da-reactions_import" class="button action">
                        <span class="dashicons dashicons-database-import" style="line-height: 1.4em"></span>
						<?php _e( 'Upload new file', 'da-reactions' ); ?>
                    </a>
                    <a href="<?= admin_url( 'admin.php' ) ?>?page=da-reactions_votes_list" class="button action">
                        <span class="dashicons dashicons-list-view" style="line-height: 1.4em"></span>
						<?php _e( 'Back to votes list', 'da-reactions' ); ?>
                    </a>
                </p>
            </div>
        </div>
		<?php
	}
	private function renderImportForm(): void {
		?>
        <div class="da-reactions-panel">
            <div class="da-reactions-panel-content">
                <h2>
					<?php _ex( 'Upload result', 'Upload result page title', 'da-reactions' ); ?>
                </h2>
                <p class="da-reactions-panel-description">
					<?php _ex(
						'In this screen you can check if everything is OK and complete import',
						'Upload result page description',
						'da-reactions'
					); ?>
                </p>
                <div class="da-reactions-panel-column-container">
                    <div class="da-reactions-panel-column">
                        <h3><?php _e( 'Uploaded file', 'da-reactions' ); ?></h3>
						<?php if ( $this->valid_csv ) { ?>
                            <p class="success">
								<?php
								echo sprintf(
									__(
										'The file was a valid CSV containing %d rows',
										'da-reactions'
									), count( $this->total_records ) ); ?></p>
						<?php } else { ?>
                            <p class="error">
								<?php
								_e( 'The file was not a valid CSV', 'da-reactions' );
								?>
                            </p>
						<?php } ?>
                    </div>
                    <div class="da-reactions-panel-column
                    <?php if ( ! $this->valid_csv ) { ?>disabled<?php } ?>">
                        <h3><?php _e( 'Invalid rows', 'da-reactions' ); ?></h3>
                        <p class="error">
							<?php
							echo sprintf(
								__(
									'There was %d invalid rows in uploaded file',
									'da-reactions'
								), count( $this->invalid_records )
							);
							?>
                        </p>
                        <a
                                download="invalid_import.csv"
                                href="<?php if ( $this->valid_csv && count( $this->invalid_records ) > 0 ) {
									echo $this->invalid_import_link;
								} else {
									echo 'javascript:;';
								} ?>">
							<?php _e( 'Download invalid data as CSV', 'da-reactions' ); ?>
                        </a>
                    </div>
                    <div
                            class="da-reactions-panel-column
                    <?php if ( ! $this->valid_csv ) { ?>disabled<?php } ?>">
                        <h3><?php _e( 'Valid rows', 'da-reactions' ); ?></h3>
                        <p class="success">
							<?php
							echo sprintf(
								__(
									'There was %d valid rows in uploaded file',
									'da-reactions'
								), count( $this->valid_records ) );
							?>
                        </p>
                        <p class="">
                            <a
                                    download="valid_import.csv"
                                    href="<?php if ( $this->valid_csv && count( $this->valid_records ) > 0 ) {
										echo $this->valid_import_link;
									} else {
										echo 'javascript:;';
									} ?>">
								<?php _e( 'Download valid data as CSV', 'da-reactions' ); ?></a>
                        </p>
						<?php if ( $this->valid_csv && count( $this->valid_records ) > 0 ) { ?>
                            <form id="importForm" action="" method="POST" enctype="multipart/form-data">
								<?php
								wp_nonce_field( $this->import_nonce, $this->import_nonce_name );
								?>
                                <input type="hidden" name="valid_import_file_name"
                                       value="<?php echo $this->valid_rows_file_name; ?>"/>
                                <input type="hidden" name="invalid_import_file_name"
                                       value="<?php echo $this->invalid_rows_file_name; ?>"/>
                                <input type="hidden" name="import_file_name"
                                       value="<?php echo $this->uploaded_file_name; ?>"/>
                                <button type="submit" class="button button-primary button-hero"
                                        href="#"><?php _e( 'Confirm import', 'da-reactions' ); ?></button>
                            </form>
						<?php } ?>
                    </div>
                </div>
            </div>
        </div>
		<?php
	}
	/**
	 * @return AdminPageImportVotes|null
	 */
	public static function getInstance(): ?AdminPageImportVotes {
		static $instance = null;
		if ( null === $instance ) {
			$instance = new self();
		}
		return $instance;
	}
	/**
	 * @param $row
	 */
	public function sanitizeRecord( $row ): void {
		if ( count( $row ) < 8 ) {
			$record = array(
				'error' => 'Wrong columns count'
			);
		} else {
			$record = array(
				'ID'            => is_numeric( $row[0] ) ? (int) $row[0] : $row[0],
				'resource_type' => sanitize_text_field( $row[1] ),
				'resource_id'   => (int) $row[2],
				'emotion_id'    => (int) $row[3],
				'user_id'       => (int) $row[4],
				'user_token'    => sanitize_text_field( $row[5] ),
				'user_ip'       => sanitize_text_field( $row[6] ),
				'created_at'    => strtotime( $row[7] ),
				'error'         => ''
			);
		}
		/// Check ID
		if ( ! $record['error'] && ! is_numeric( $record['ID'] ) ) {
			/// First line
			$record['error'] = 'First column is not numeric';
		}
		/// Check Content Type
		if ( ! $record['error'] ) {
			if (
				! in_array( $record['resource_type'], $this->unregistered_types, true ) &&
				in_array( $record['resource_type'], $this->registered_types, true ) &&
				(
					post_type_exists( $record['resource_type'] ) ||
					$record['resource_type'] === 'comment'
				)
			) {
				$this->registered_types[] = $record['resource_type'];
			} else {
				$this->unregistered_types[] = $record['resource_type'];
				$record['error']            = 'Invalid Resource Type';
			}
		}
		/// Check Content by ID
		if ( ! $record['error'] ) {
			$content_name_and_id = "${record['resource_type']}___${record['resource_id']}";
			if (
				$record['resource_type'] === 'comment' &&
				! in_array( $content_name_and_id, $this->unregistered_contents, true ) &&
				! in_array( $content_name_and_id, $this->registered_contents, true )
			) {
				$comment = get_comment( $record['resource_id'] );
				if ( ! $comment ) {
					$this->unregistered_contents[] = $content_name_and_id;
					$record['error']               = 'Comment not found';
				} else {
					$this->registered_contents[] = $content_name_and_id;
				}
			} else {
				$post = get_post( $record['resource_id'] );
				if ( ! $post || $post->post_type !== $record['resource_type'] ) {
					$record['error'] = 'Post not found';
				}
			}
		}
		/// Check Reaction By ID
		if ( ! $record['error'] && ! in_array( $record['emotion_id'], $this->unregistered_reactions, true ) &&
		     ! in_array( $record['emotion_id'], $this->registered_reactions, true ) ) {
			$reaction = Data::getReactionById( $record['emotion_id'] );
			if ( $reaction ) {
				$this->registered_reactions[] = $record['emotion_id'];
			} else {
				$record['error']                = 'Reaction not found';
				$this->unregistered_reactions[] = $record['emotion_id'];
			}
		}
		/// Check User By ID
		if ( ! $record['error'] ) {
			if (
				(int) $record['user_id'] !== 0 &&
				! in_array( $record['user_id'], $this->unregistered_users, true ) &&
				! in_array( $record['user_id'], $this->registered_users, true )
			) {
				$user = get_userdata( $record['user_id'] );
				if ( ! $user ) {
					$this->unregistered_users[] = $record['user_id'];
					$record['error']            = 'User not found';
				} else {
					$this->registered_users[] = $record['user_id'];
				}
			}
			$this->valid_records[] = $record;
		} else {
			$this->invalid_records[] = $record;
		}
		$this->total_records[] = $record;
	}
}
